@@ -1,7 +1,7 @@
 //! _msgpack-schema_ is a schema language for describing data formats encoded in MessagePack.
 //! It provides two derive macros `Serialize` and `Deserialize` that allow you to transcode MessagePack binary data to/from Rust data structures in a type-directed way.
 //!
-//! ```rust
+//! ```rust,ignore
 //! use msgpack_schema::{Deserialize, Serialize};
 //!
 //! #[derive(Deserialize, Serialize)]
@@ -41,7 +41,7 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! struct S {
 //!     #[tag = 0]
 //!     foo: u32,
@@ -53,7 +53,7 @@
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! S { foo: 42, bar: "hello".to_owned() }
 //! ```
 //!
@@ -94,14 +94,14 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! struct S(u32)
 //! ```
 //!
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! S(42)
 //! ```
 //!
@@ -136,14 +136,14 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! struct S
 //! ```
 //!
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! S
 //! ```
 //!
@@ -157,14 +157,14 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! struct S()
 //! ```
 //!
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! S()
 //! ```
 //!
@@ -197,7 +197,7 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! enum E {
 //!     #[tag = 3]
 //!     Foo
@@ -207,7 +207,7 @@
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! E::Foo
 //! ```
 //!
@@ -224,7 +224,7 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! enum E {
 //!     #[tag = 3]
 //!     Foo()
@@ -234,7 +234,7 @@
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! E::Foo()
 //! ```
 //!
@@ -269,7 +269,7 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! enum E {
 //!     #[tag = 3]
 //!     Foo(u32)
@@ -279,7 +279,7 @@
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! E::Foo(42)
 //! ```
 //!
@@ -316,7 +316,7 @@
 //! <tr>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! #[derive(Serialize, Deserialize)]
 //! #[untagged]
 //! enum Animal {
@@ -328,7 +328,7 @@
 //! </td>
 //! <td>
 //!
-//! ```rust
+//! ```rust,ignore
 //! E::Bar(42)
 //! ```
 //!
@@ -877,7 +877,7 @@ impl<T: Deserialize> Deserialize for Vec<T> {
     }
 }
 
-pub struct BinarySerializer<W> {
+struct BinarySerializer<W> {
     w: W,
 }
 
@@ -953,7 +953,7 @@ trait ReadExt: ReadBytesExt {
 
 impl<R: io::Read> ReadExt for R {}
 
-pub struct BinaryDeserializer<R> {
+struct BinaryDeserializer<R> {
     r: R,
 }
 
@@ -1033,11 +1033,11 @@ impl<R: io::Read> Deserializer for BinaryDeserializer<R> {
             }
             rmp::Marker::FixExt8 => {
                 let tag = self.r.read_i8()?;
-                Token::Ext(tag, self.r.read_to_vec(16)?)
+                Token::Ext(tag, self.r.read_to_vec(8)?)
             }
             rmp::Marker::FixExt16 => {
                 let tag = self.r.read_i8()?;
-                Token::Ext(tag, self.r.read_to_vec(1)?)
+                Token::Ext(tag, self.r.read_to_vec(16)?)
             }
             rmp::Marker::Ext8 => {
                 let len = self.r.read_u8()? as usize;
@@ -1066,6 +1066,47 @@ pub fn deserialize<D: Deserialize, R: io::Read>(r: R) -> Result<D, DeserializeEr
     Ok(D::deserialize(&mut deserializer)?)
 }
 
+/// Constructs a MessagePack object.
+///
+/// Return type is an opaque type implementing [Serialize].
+///
+/// # Example
+///
+/// ```
+/// # use msgpack_schema::{msgpack, value::Bin, Serialize, serialize};
+/// msgpack!(
+///   // array literal
+///   [
+///     // numeric literals
+///     0,
+///     -42,
+///     3.14,
+///     2.71f32,
+///     // actually any expression is allowed (as long as it's a supported type)
+///     1 + 2 + 3,
+///     // boolean
+///     true,
+///     false,
+///     // nil is a keyword denoting the nil object
+///     nil,
+///     // map object
+///     { "any value in key": nil },
+///     { 0: 1, "trailing comma is ok": nil, },
+///     // string literal to make a string object
+///     "hello",
+///     // Use an expression of [Bin] type to create a binary object
+///     Bin(vec![0xDE, 0xAD, 0xBE, 0xEF])
+///   ]
+/// )
+/// # ;
+/// ```
+#[macro_export]
+macro_rules! msgpack {
+    ( $( $tt: tt )+ ) => {
+        $crate::msgpack_value!($( $tt )+)
+    };
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1138,4 +1179,21 @@ mod tests {
 
         assert_eq!(val, value::deserialize(value::serialize(&val)).unwrap())
     }
+
+    #[test]
+    fn msgpack_macro() {
+        let val = Human {
+            age: 42,
+            name: "John".into(),
+        };
+        let mut buf1 = vec![];
+        serialize(&val, &mut buf1).unwrap();
+        let lit = msgpack!({
+            0: 42,
+            1: "John",
+        });
+        let mut buf2 = vec![];
+        serialize(&lit, &mut buf2).unwrap();
+        assert_eq!(buf1, buf2);
+    }
 }
